==> jfrog-crypto-3.10.2-sources.jar: org/jfrog/security/ssl/CertificateHelper.java <==
package org.jfrog.security.ssl;

import org.apache.commons.codec.digest.DigestUtils;
import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x500.RDN;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x500.style.BCStyle;
import org.bouncycastle.asn1.x500.style.IETFUtils;
import org.bouncycastle.asn1.x509.Extension;
import org.bouncycastle.asn1.x509.GeneralName;
import org.bouncycastle.asn1.x509.GeneralNames;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.cert.CertIOException;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.X509v3CertificateBuilder;
import org.bouncycastle.cert.jcajce.JcaX509CertificateHolder;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.bouncycastle.x509.extension.X509ExtensionUtil;
import org.jfrog.security.util.BCProviderFactory;

import javax.security.auth.x500.X500Principal;
import javax.security.cert.X509Certificate;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.CertificateEncodingException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.UUID;

/**
 * @author Yinon Avraham.
 */
public abstract class CertificateHelper {



    private CertificateHelper() {
        // utility class should not be instantiated
    }

    public static final ASN1ObjectIdentifier CERT_VERSION_OID = (new ASN1ObjectIdentifier("2.5.29.17.1")).intern();
    private static final String SIG_ALG = "SHA256WithRSA";
    private static final String BC_PROVIDER = "BC";
    private static final long ONE_DAY_IN_MS = 24 * 60 * 60 * 1000L;
    private static final long MAX_EXPIRY;
    static {
        Calendar calendar = Calendar.getInstance();
        calendar.set(7000, 0, 1, 0, 0); // We'll need to fix it once the year 7000 will be around the corner...
        MAX_EXPIRY = calendar.getTimeInMillis();
        BCProviderFactory.getProvider(); // Make sure that the factory is initialised
    }

    public static class CryptoInfo{
        private KeyPair keyPair;
        private java.security.cert.X509Certificate certificate;

        public CryptoInfo(KeyPair keyPair, java.security.cert.X509Certificate certificate){
            this.keyPair = keyPair;
            this.certificate = certificate;
        }

        public KeyPair getKeyPair() {
            return keyPair;
        }

        public void setKeyPair(KeyPair keyPair) {
            this.keyPair = keyPair;
        }

        public java.security.cert.X509Certificate getCertificate() {
            return certificate;
        }

        public void setCertificate(java.security.cert.X509Certificate certificate) {
            this.certificate = certificate;
        }
    }

    public static X509Certificate generateSignedCertificate(X500Principal issuer,
            PrivateKey issuerPrivateKey, X500Principal subject, PublicKey subjectPublicKey,
            BigInteger serialNumber, long expireIn) throws CertificateGenerationException {
        return generateSignedCertificate(issuer, issuerPrivateKey, subject, subjectPublicKey, serialNumber, expireIn, false, 0);
    }

    public static X509Certificate generateSignedCertificate(X500Principal issuer,
            PrivateKey issuerPrivateKey, X500Principal subject, PublicKey subjectPublicKey,
            BigInteger serialNumber, long expireIn, boolean addSubAltNames, int certVersion) throws CertificateGenerationException {
        try {
            Date startDate = new Date(System.currentTimeMillis() - ONE_DAY_IN_MS);
            if (expireIn < 0) {
                throw new IllegalArgumentException("'expire in' must be a positive number");
            }
            long endMillis = System.currentTimeMillis() + expireIn;
            //Handle possible overflow or too large numbers
            endMillis = (endMillis < 0 || endMillis > MAX_EXPIRY) ? MAX_EXPIRY : endMillis;
            Date endDate = new Date(endMillis);

            X509v3CertificateBuilder builder = new X509v3CertificateBuilder(
                    X500Name.getInstance(issuer.getEncoded()),
                    serialNumber,
                    startDate,
                    endDate,
                    X500Name.getInstance(subject.getEncoded()),
                    SubjectPublicKeyInfo.getInstance(new ASN1InputStream(subjectPublicKey.getEncoded()).readObject()));

            if (addSubAltNames)
                setSubAltNames(subject, builder);
            addCertificateVersion(builder, certVersion);
            ContentSigner sigGen = new JcaContentSignerBuilder(SIG_ALG).setProvider(BC_PROVIDER).build(issuerPrivateKey);
            X509CertificateHolder certHolder = builder.build(sigGen);

            return X509Certificate.getInstance(certHolder.getEncoded());
        } catch (Exception e) {
            throw new CertificateGenerationException("Failed to generate signed certificate: " + e.getMessage(), e);
        }
    }

    public static int getCertificateVersion(java.security.cert.X509Certificate certificate) throws IOException {
        byte[] version = certificate.getExtensionValue(CERT_VERSION_OID.getId());
        if (version != null) {
            Integer versionFromCertificate = null;
            GeneralNames gn = GeneralNames.getInstance(X509ExtensionUtil.fromExtensionValue(version));
            GeneralName[] names = gn.getNames();
            for (GeneralName name : names) {
                if (name.getTagNo() == GeneralName.otherName) {
                    ASN1Sequence seq = ASN1Sequence.getInstance(name.getName());
                    ASN1Integer value = (ASN1Integer) seq.getObjectAt(1);
                    versionFromCertificate = value.getValue().intValue();
                }
            }
            return versionFromCertificate;
        }
        return 0;
    }


    public static java.security.cert.X509Certificate generateRootX509Certificate(
            KeyPair keyPair,
            String commonName ,
            boolean addSubAltNames,
            int certVersion) throws CertificateGenerationException {
        X509Certificate cert = generateRootCertificate(keyPair, commonName, addSubAltNames, certVersion);
        return convert(cert);
    }

    public static javax.security.cert.X509Certificate generateRootCertificate(
            KeyPair keyPair,
            String commonName ,
            boolean addSubAltNames,
            int certVersion) throws CertificateGenerationException {
        BigInteger serialNumber = randomSerialNumber();
        X500Principal accessPrincipal = new X500Principal("CN=" + commonName);

        return CertificateHelper.generateSignedCertificate(
                accessPrincipal, keyPair.getPrivate(),
                accessPrincipal, keyPair.getPublic(),
                serialNumber, Long.MAX_VALUE, addSubAltNames, certVersion);
    }

    // Converts to java.security
    public static java.security.cert.X509Certificate convert(javax.security.cert.X509Certificate cert) {
        try {
            byte[] encoded = cert.getEncoded();
            ByteArrayInputStream bis = new ByteArrayInputStream(encoded);
            java.security.cert.CertificateFactory cf
                    = java.security.cert.CertificateFactory.getInstance("X.509");
            return (java.security.cert.X509Certificate)cf.generateCertificate(bis);
        } catch (java.security.cert.CertificateEncodingException e) {
        } catch (javax.security.cert.CertificateEncodingException e) {
        } catch (java.security.cert.CertificateException e) {
        }
        return null;
    }

    private static BigInteger randomSerialNumber() {
        UUID uuid = UUID.randomUUID();
        ByteBuffer bb = ByteBuffer.wrap(new byte[16]);
        bb.putLong(uuid.getMostSignificantBits());
        bb.putLong(uuid.getLeastSignificantBits());
        return new BigInteger(1,bb.array());
    }


    private static String addSchema(String subjectName) {
        return "http://"+subjectName;
    }

    private static void setSubAltNames(X500Principal subject, X509v3CertificateBuilder builder) throws CertIOException {
        String[] serverId = subject.getName().split("@");
        if (serverId.length == 2) {
            String parsedSubjAltName = serverId[1];
            GeneralName altName = new GeneralName(GeneralName.dNSName, parsedSubjAltName);
            GeneralNames subjectAltName = new GeneralNames(altName);
            builder.addExtension(Extension.subjectAlternativeName, false, subjectAltName);
        }
    }

    private static void addCertificateVersion(X509v3CertificateBuilder builder, int version)
            throws CertIOException {


        DERSequence seq = new DERSequence(new ASN1Encodable[] { CERT_VERSION_OID, new ASN1Integer(version) });

        ArrayList<GeneralName> namesList = new ArrayList<>();
        namesList.add(new GeneralName(GeneralName.otherName, seq));
        GeneralNames versionName = GeneralNames.getInstance(new DERSequence((GeneralName[]) namesList.toArray(new GeneralName[] {})));


        builder.addExtension(CERT_VERSION_OID, false, versionName);
    }

    /**
     * Get the common name (CN) of the issuer of a given certificate
     *
     * @param certificate the certificate
     * @return the common name
     *
     * @throws CertificateEncodingException
     */
    public static String getCertificateIssuerCommonName(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getIssuer().getRDNs(BCStyle.CN);
        if (rdNs != null && rdNs.length > 0) {
            RDN commonName = rdNs[0];
            if (commonName.getFirst() != null && commonName.getFirst().getValue() != null) {
                result = IETFUtils.valueToString(commonName.getFirst().getValue());
            }
        }
        return result;
    }

    public static String getCertificateIssuerOrganizationUnit(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getIssuer().getRDNs(BCStyle.OU);
        if (rdNs != null && rdNs.length > 0) {
            RDN organizationUnit = rdNs[0];
            if (organizationUnit.getFirst() != null && organizationUnit.getFirst().getValue() != null) {
                result = IETFUtils.valueToString(organizationUnit.getFirst().getValue());
            }
        }
        return result;
    }

    public static String getCertificateIssuerOrganization(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getIssuer().getRDNs(BCStyle.O);
        if (rdNs != null && rdNs.length > 0) {
            RDN organization = rdNs[0];
            if (organization.getFirst() != null && organization.getFirst().getValue() != null) {
                result = IETFUtils.valueToString(organization.getFirst().getValue());
            }
        }
        return result;
    }

    public static String getCertificateSubjectCommonName(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getSubject().getRDNs(BCStyle.CN);
        if (rdNs != null && rdNs.length > 0) {
            RDN commonName = rdNs[0];
            if (commonName.getFirst() != null && commonName.getFirst().getValue() != null) {
                result = IETFUtils.valueToString(commonName.getFirst().getValue());
            }
        }
        return result;
    }

    public static String getCertificateSubjectOrganizationUnit(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getSubject().getRDNs(BCStyle.OU);
        if (rdNs != null && rdNs.length > 0) {
            RDN organizationUnit = rdNs[0];
            if (organizationUnit.getFirst() != null && organizationUnit.getFirst().getValue() != null) {
                result = IETFUtils.valueToString(organizationUnit.getFirst().getValue());
            }
        }
        return result;
    }

    public static String getCertificateSubjectOrganization(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getSubject().getRDNs(BCStyle.O);
        if (rdNs != null && rdNs.length > 0) {
            RDN organization = rdNs[0];
            if (organization.getFirst() != null && organization.getFirst().getValue() != null) {
                result = IETFUtils.valueToString(organization.getFirst().getValue());
            }
        }
        return result;
    }

    public static Date getValidUntil(java.security.cert.X509Certificate certificate) {
        return certificate.getNotAfter();
    }

    public static Date getIssuedAt(java.security.cert.X509Certificate certificate) {
        return certificate.getNotBefore();
    }

    public static String getCertificateFingerprint(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        byte[] bytes = DigestUtils.sha256(certificate.getEncoded());
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < bytes.length; i++) {
            String appendString = Integer.toHexString(0xFF & bytes[i]);
            if (appendString.length() == 1) {
                builder.append("0");
            }
            builder.append(appendString.toUpperCase());
            if (i != bytes.length - 1) {
                builder.append(':');
            }
        }
        return builder.toString();
    }
}

==> jfrog-crypto-4.4.6-sources.jar: org/jfrog/security/ssl/CertificateHelper.java <==
package org.jfrog.security.ssl;

import org.apache.commons.codec.digest.DigestUtils;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.x500.RDN;
import org.bouncycastle.asn1.x500.style.BCStyle;
import org.bouncycastle.asn1.x500.style.IETFUtils;
import org.bouncycastle.asn1.x509.Extension;
import org.bouncycastle.asn1.x509.Extensions;
import org.bouncycastle.asn1.x509.GeneralName;
import org.bouncycastle.asn1.x509.GeneralNames;
import org.bouncycastle.cert.jcajce.JcaX509CertificateHolder;
import org.bouncycastle.x509.extension.X509ExtensionUtil;
import org.jfrog.security.util.BCProviderFactory;

import java.io.IOException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.security.KeyPair;
import java.security.cert.CertificateEncodingException;
import java.util.Date;
import java.util.UUID;

/**
 * @author Yinon Avraham.
 */
public abstract class CertificateHelper {

    private CertificateHelper() {
        // utility class should not be instantiated
    }

    public static final ASN1ObjectIdentifier CERT_VERSION_OID = (new ASN1ObjectIdentifier("2.5.29.17.1")).intern();

    static {
        BCProviderFactory.getProvider(); // Make sure that the factory is initialised
    }

    public static class CryptoInfo {
        private KeyPair keyPair;
        private java.security.cert.X509Certificate certificate;

        public CryptoInfo(KeyPair keyPair, java.security.cert.X509Certificate certificate) {
            this.keyPair = keyPair;
            this.certificate = certificate;
        }

        public KeyPair getKeyPair() {
            return keyPair;
        }

        public void setKeyPair(KeyPair keyPair) {
            this.keyPair = keyPair;
        }

        public java.security.cert.X509Certificate getCertificate() {
            return certificate;
        }

        public void setCertificate(java.security.cert.X509Certificate certificate) {
            this.certificate = certificate;
        }
    }

    public static int getCertificateVersion(java.security.cert.X509Certificate certificate) throws IOException {
        byte[] version = certificate.getExtensionValue(CERT_VERSION_OID.getId());
        if (version != null) {
            Integer versionFromCertificate = null;
            GeneralNames gn = GeneralNames.getInstance(X509ExtensionUtil.fromExtensionValue(version));
            GeneralName[] names = gn.getNames();
            for (GeneralName name : names) {
                if (name.getTagNo() == GeneralName.otherName) {
                    ASN1Sequence seq = ASN1Sequence.getInstance(name.getName());
                    ASN1Integer value = (ASN1Integer) seq.getObjectAt(1);
                    versionFromCertificate = value.getValue().intValue();
                }
            }
            return versionFromCertificate;
        }
        return 0;
    }

    public static java.security.cert.X509Certificate generateRootCertificate(
            KeyPair keyPair,
            String commonName,
            int certVersion) throws CertificateGenerationException {
        BigInteger serialNumber = randomSerialNumber();

        return SignedCertificateBuilder.builder()
                .iss(commonName)
                .issPrivateKey(keyPair.getPrivate())
                .sub(commonName)
                .subPublicKey(keyPair.getPublic())
                .serialNumber(serialNumber)
                .expireIn(Long.MAX_VALUE)
                .certVersion(certVersion)
                .useSubForSAN()
                .isCA(true)
                .buildX509Certificate();

    }

    public static BigInteger randomSerialNumber() {
        UUID uuid = UUID.randomUUID();
        ByteBuffer bb = ByteBuffer.wrap(new byte[16]);
        bb.putLong(uuid.getMostSignificantBits());
        bb.putLong(uuid.getLeastSignificantBits());
        return new BigInteger(1, bb.array());
    }

    /**
     * Get the common name (CN) of the issuer of a given certificate
     *
     * @param certificate the certificate
     * @return the common name
     * @throws CertificateEncodingException
     */
    public static String getCertificateIssuerCommonName(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getIssuer().getRDNs(BCStyle.CN);
        return getName(result, rdNs);
    }

    /**
     * Get the common name (CN) of the issuer of a given certificate
     *
     * @param certificate the certificate
     * @return the common name
     * @throws CertificateEncodingException
     */
    public static String getCertificateSubjectAlternativeName(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        Extensions extensions = certificateHolder.getExtensions();
        GeneralNames generalNames = GeneralNames.fromExtensions(extensions, Extension.subjectAlternativeName);
        if (generalNames == null) {
            return "";
        }
        GeneralName[] names = generalNames.getNames();
        if (names == null || names.length == 0) {
            return "";
        }
        GeneralName generalName = names[0]; //since we only put in one name
        return generalName.getName().toString();

    }

    public static String getCertificateSubjectCommonName(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getSubject().getRDNs(BCStyle.CN);
        return getName(result, rdNs);
    }

    public static Date getIssuedAt(java.security.cert.X509Certificate certificate) {
        return certificate.getNotBefore();
    }

    public static String getCertificateIssuerOrganizationUnit(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getIssuer().getRDNs(BCStyle.OU);
        return getName(result, rdNs);
    }

    public static String getCertificateIssuerOrganization(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getIssuer().getRDNs(BCStyle.O);
        return getName(result, rdNs);
    }

    public static String getCertificateSubjectOrganizationUnit(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getSubject().getRDNs(BCStyle.OU);
        return getName(result, rdNs);
    }

    public static String getCertificateSubjectOrganization(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        String result = "Unknown";
        JcaX509CertificateHolder certificateHolder = new JcaX509CertificateHolder(certificate);
        RDN[] rdNs = certificateHolder.getSubject().getRDNs(BCStyle.O);
        return getName(result, rdNs);
    }

    public static Date getValidUntil(java.security.cert.X509Certificate certificate) {
        return certificate.getNotAfter();
    }

    public static String getCertificateFingerprint(java.security.cert.X509Certificate certificate)
            throws CertificateEncodingException {
        byte[] bytes = DigestUtils.sha256(certificate.getEncoded());
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < bytes.length; i++) {
            String appendString = Integer.toHexString(0xFF & bytes[i]);
            if (appendString.length() == 1) {
                builder.append("0");
            }
            builder.append(appendString.toUpperCase());
            if (i != bytes.length - 1) {
                builder.append(':');
            }
        }
        return builder.toString();
    }

    private static String getName(String result, RDN[] rdNs) {
        if (rdNs != null && rdNs.length > 0) {
            RDN firstRDN = rdNs[0];
            if (firstRDN.getFirst() != null && firstRDN.getFirst().getValue() != null) {
                result = IETFUtils.valueToString(firstRDN.getFirst().getValue());
            }
        }
        return result;
    }
}
